      SUBROUTINE OPTTRJ(N,XEQUI,X0,I0,R,TOL,CUT,KPRINT,IERR,
     &   RWORK,LRWORK,IWORK,LIWORK)
C                                                                       
C  THIS SUBROUTINE SOLVES BOUNDARY VALUE PROBLEMS ON [0,INF) E.G. ARISING
C  FROM OPTIMAL CONTROL PROBLEMS WITH INFINITE TIME HORIZON 
C
C              X'=F(X), A*X(0)=A*X0, X(INF)=XEQUI
C
C  INCLUDING THE DETERMINATION OF AN EQUILIBRIUM XEQUI AND THE EIGENVALUES
C  OF THE LINEARIZATION AT THE EQUILIBRIUM. IT SERVES AS EASY-TO-USE
C  INTERFACE TO SOPHISTICATED PUBLIC DOMAIN SOFTWARE.
C
C                                             VERSION  16.02.1998       
C                                                                       
C                                                                       
C  NOTE FOR COMPILATION
C
C  THE INCLUDED SOFTWARE PACKAGES REQUIRE STATIC STORAGE FOR VARIABLES
C  AND COMMON BLOCKS. THEREFORE APPROPRIATE COMIPLER FLAGS MUST BE SET.
C                                                                       
C                                                                       
C  REFERENCES                                                           
C                                                                       
C    /1/  P. KUNKEL, O. VON DEM HAGEN:                          
C         NUMERICAL SOLUTION OF INFINITE HORIZON OPTIMAL CONTROL PROBLEMS
C         SUBMITTED FOR PUBLICATION
C                                                                       
C                                                                       
C  INPUT PARAMETERS  (* MARKS TRANSIENT PARAMETERS)                     
C                                                                       
C    - N          NUMBER OF DIFFERENTIAL EQUATIONS  
C                 (MAX ALLOWED NUMBER IS 32)     
C                                                                       
C    - XEQUI(N) * ESTIMATE FOR EQUILIBRIUM
C                                                              
C    - X0(N)      INITIAL VALUES FOR THE DIFFERENTIAL EQUATION
C                                                                       
C    - I0(N)      INDEXATION OF THE INITIAL VALUES X0
C
C                 THE NUMBER OF POSSIBLE INITIAL VALUES EQUALS
C                 THE NUMBER OF STABLE EIGENVALUES OF THE EQUILIBRIUM.
C                 IF THE NUMBER OF STABLE EIGENVALUES IS NS, THE CODE
C                 USES THE INITIAL CONDITIONS
C                       X_I0(I)(0)=X0_I0(I), I=1,...,NS.
C                                                                       
C    - R        * PARAMETER FOR THE TRANSFORMATION OF [0,INF) TO [0,1]
C                 BY S=EXP(R*T). IF THE VALUE IS NOT NEGATIVE, THE CODE
C                 TRIES TO CHOOSE AN APPROPRIATE VALUE. THE MODULUS OF R
C                 SHOULD BE SMALL ENOUGH TO GIVE RISE TO A SUFFICIENTLY
C                 SMOOTH SOLUTION ON [0,1].
C                                                                       
C    - TOL        TOLERANCE USED IN THE PUBLIC DOMAIN SOFTWARE FOR
C                 SOLVING NONLINEAR EQUATIONS AND BOUNDARY VALUE PROBLEMS
C                                                                       
C    - CUT        TOLERANCE USED IN THE CLASSIFICATION OF THE EIGENVALUES.
C                 NUMBERS SMALLER THAN CUT IN MODULUS ARE TREATED AS ZERO.
C                                                                       
C    - KPRINT     PRINT PARAMETER (OUTPUT UNIT IS 6)
C                 KPRINT=0: NO OUPUT
C                 KPRINT=1: SMALL OUPUT
C                 KPRINT=2: LARGE OUPUT
C                                                                       
C    - RWORK      REAL WORKSPACE                
C                                                                       
C    - LRWORK     LENGTH OF REAL WORKSPACE                
C                                                                       
C    - IWORK      INTEGER WORKSPACE                         
C                                                                       
C    - LIWORK     LENGTH OF INTEGER WORKSPACE 
C
C                 SEE SUBROUTINES NLEQ1 AND COLCON FOR DETAILS ON THE
C                 REQUIRED SIZE OF THE WORKSPACES
C                  
C                                                                       
C  SUBROUTINES TO BE SUPPLIED BY THE USER 
C   
C  THE USER MUST SUPPLY SUBROUTINES FOR THE EVALUATION OF F AND ITS
C  JACOBIAN DFDX AT A GIVEN POINT X. THEY MUST HAVE THE FOLLOWING FORM.
C
C      SUBROUTINE FEVAL(N,X,F)
C      IMPLICIT REAL*8(A-H,O-Z)
C      DIMENSION X(N),F(N)
C        .
C        .
C        .
C      RETURN
C      END
C
C      SUBROUTINE JEVAL(N,L,X,DFDX)
C      IMPLICIT REAL*8(A-H,O-Z)
C      DIMENSION X(N),DFDX(L,N)
C        .
C        .
C        .
C      RETURN
C      END      
C 
C  THE PARAMETERS N, L, AND X ARE INPUT AND MUST NOT BE ALTERED BY
C  THE SUBROUTINES. ON RETURN F AND DFDX MUST CONTAIN F(X) AND F_X(X)
C  RESPECTIVELY. COMPARE WITH THE DEMONSTRATION DRIVERS INCLUDED
C  IN THIS PACKAGE. NOTE THAT YOU MUST USE FEVAL AND JEVAL AS NAMES
C  OF THESE SUBROUTINES.
C
C                                                                       
C  OUTPUT PARAMETERS                                                    
C                                                                    
C    - XEQUI(N)   COMPUTED EQUILIBRIUM
C                                                    
C    - IERR       ERROR EXIT PARAMETER            
C                   0  NO ERROR OCCURRED    
C                   1  INVALID INPUT
C                   2  DETERMINATION OF EQUILIBRIUM FAILED
C                   3  DETERMINATION OF EIGENVALUES FAILED         
C                   4  NONHYPERBOLIC EQUILIBRIUM FOUND   
C                   5  INVERSION OF EIGENVECTOR MATRIX FAILED     
C                   6  BOUNDARY VALUE PROBLEM SOLVER FAILED
C
C    ON RETURN THE INFORMATION ON A SUCCESSFULLY COMPUTED SOLUTION 
C    OF THE BOUNDARY VALUE PROBLEM IS STORED IN THE WORKSPACES.
C    FUNCTION VALUES CAN BE OBTAINED BY CALLING THE SUBROUTINE APPSLN
C    WHICH IS PART OF THE COLCON PACKAGE. SINCE THE HOMOTOPY PARAMETER
C    IS PART OF THIS SOLUTION, THE SOLUTION VECTOR MUST BE DIMENSIONED
C    AS X(N1) WHERE N1=N+1. FOR DETAILS SEE THE MANUAL CONTAINED IN
C    THE COLCON PACKAGE.
C
C
C    PACKAGES USED IN THIS IMPLEMENTATION
C
C    THIS IMPLEMENTATION USES A NUMBER OF PUBLIC DOMAIN SOFTWARE PACKAGES.
C
C    - NLEQ1      SOLVER FOR SYSTEMS OF NONLINEAR EQUATIONS
C
C                 THIS PACKAGE IS PART OF THE CODELIB LIBRARY AND CAN 
C                 BE OBTAINED E.G. VIA
C
C                   ftp://elib.zib-berlin.de/pub/elib/codelib/nleq1/
C
C                 NOTE THAT THIS PACKAGE INCLUDES A SUBROUTINE D1MACH
C                 WHERE MACHINE DEPENDENT CONSTANTS MUST BE ADAPTED
C                 TO THE USED HARDWARE
C
C    - BALANC BALBAK CDIV HQR2 ORTBAK ORTHES
C                 SUITE OF SUBROUTINES FOR COMPUTING EIGENVALUES AND
C                 EIGENVECTORS OF A GENERAL REAL MATRIX
C
C                 THESE SUBROUTINES ARE PART OF THE EISPACK LIBRARY
C                 WITHIN THE NETLIB LIBRARY AND CAN BE OBTAINED E.G. VIA
C
C                   ftp://elib.zib-berlin.de/pub/netlib/eispack/
C
C    - COLCON     PACKAGE FOR SOLVING BOUNDARY VALUE PROBLEMS
C
C                 THIS PACKAGE CAN BE OBTAINED BY REQUEST FROM
C
C                   mailto://kunkel@math.uni-oldenburg.de
C
C    THE IMPLEMENTATION IS BASED ON LINEAR ALGEBRA SUBROUTINES FROM THE
C    LINPACK AND BLAS LIBRARY. THESE CAN BE FOUND E.G. AT
C
C                   ftp://elib.zib-berlin.de/pub/netlib/linpack/
C    AND
C                   ftp://elib.zib-berlin.de/pub/netlib/blas/
C 
      IMPLICIT REAL*8(A-H,O-Z)
      PARAMETER (NMAX=32,NMAX1=NMAX+1)
      DIMENSION XEQUI(N),X0(N),I0(N)
      DIMENSION RWORK(LRWORK),IWORK(LIWORK)
      DIMENSION XFIN(NMAX),X0COM(NMAX),I0COM(NMAX)
      DIMENSION WR(NMAX),WI(NMAX),Z(NMAX,NMAX),ZINV(NMAX,NMAX)
      DIMENSION XSCAL(NMAX),IOPT(50)
      DIMENSION A(NMAX,NMAX),SCALE(NMAX)
      DIMENSION ORT(NMAX),H(NMAX,NMAX),IPVT(NMAX)
      DIMENSION ZETA(NMAX),ZTOL(NMAX),FIXPNT(1),M(NMAX),LTOL(NMAX)
      DIMENSION INFO(16),WINDOW(3)
      EXTERNAL FCN,JAC
      EXTERNAL FSUB,DFSUB,GSUB,DGSUB,GUESS
      COMMON /COMPRM/ XFIN,X0COM,WR,WI,Z,ZINV,RCOM,I0COM,NS,NU,NCOM
      DATA IOPT/50*0/
C.. CHECK INPUT
      IERR=0
      IF (N.LE.0 .OR. N.GT.NMAX) IERR=1
      IF (TOL.LE.0.D0) IERR=1
      IF (CUT.LE.0.D0) IERR=1
      IF (IERR.NE.0) THEN
      IF (KPRINT.GT.0) WRITE(6,60909)
60909 FORMAT('INVALID INPUT')
      RETURN
      END IF
C.. COMPUTE EQUILIBRIUM 
      RTOL=TOL
      IOPT(3)=1
      IOPT(11)=KPRINT
      IOPT(13)=KPRINT
      IOPT(15)=KPRINT
      IOPT(31)=3
      DO 100 I=1,N
      XSCAL(I)=1.D0
  100 CONTINUE
      DO 105 I=1,50
      RWORK(I)=0.D0
      IWORK(I)=0
  105 CONTINUE
      CALL NLEQ1(N,FCN,JAC,XEQUI,XSCAL,RTOL,IOPT,IERR,
     &   LIWORK,IWORK,LRWORK,RWORK)
      IF (IERR.GT.0) THEN
      IERR=2
      RETURN
      END IF
C.. DETERMINE EIGENVALUES OF JACOBIAN                               
      CALL JEVAL(N,NMAX,XEQUI,A) 
      IF (KPRINT.GT.1) THEN
      WRITE(6,60101) 
      DO 110 I=1,N
      WRITE(6,60111) I,(A(I,J),J=1,N)
  110 CONTINUE
60101 FORMAT(/'JACOBIAN AT EQUILIBRIUM')
60111 FORMAT('ROW ',I2/(4D18.10))
      END IF
      CALL BALANC(NMAX,N,A,LOW,IGH,SCALE)
      CALL ORTHES(NMAX,N,LOW,IGH,A,ORT)
      DO 115 I=1,N
      DO 115 J=1,N
      H(I,J)=A(I,J)
  115 CONTINUE
      DO 117 I=1,N      
      DO 116 J=1,N
      Z(I,J)=0.D0
  116 CONTINUE
      Z(I,I)=1.D0
  117 CONTINUE
      IF (KPRINT.GT.1) THEN
      WRITE(6,60102) 
      DO 120 I=1,N
      WRITE(6,60121) I,(H(I,J),J=1,N)
  120 CONTINUE
60102 FORMAT(/'HESSENBERG FORM OF JACOBIAN')
60121 FORMAT('ROW ',I2/(4D18.10))
      END IF
      CALL HQR2(NMAX,N,LOW,IGH,H,WR,WI,Z,IERR)
      IF (IERR.NE.0) THEN
      IERR=3
      IF (KPRINT.GT.0) WRITE(6,60908)
60908 FORMAT('EIGENVALUE COMPUTATION FAILED')
      RETURN
      END IF
      IF (KPRINT.GT.0) THEN
      WRITE(6,60103) 
      DO 130 I=1,N
      WRITE(6,60131) I,WR(I),WI(I)
  130 CONTINUE
60103 FORMAT(/'EIGENVALUES')
60131 FORMAT('NO. ',I2/2D18.10)
      END IF
      IF (KPRINT.GT.1) THEN
      WRITE(6,60104) 
      DO 140 I=1,N
      WRITE(6,60141) I,(Z(I,J),J=1,N)
  140 CONTINUE
60104 FORMAT(/'TRANSFORMED EIGENVECTOR MATRIX')
60141 FORMAT('ROW ',I2/(4D18.10))
      END IF
      CALL ORTBAK(NMAX,LOW,IGH,A,ORT,N,Z)
      CALL BALBAK(NMAX,N,LOW,IGH,SCALE,N,Z)
      IF (KPRINT.GT.1) THEN
      WRITE(6,60105) 
      DO 150 I=1,N
      WRITE(6,60151) I,(Z(I,J),J=1,N)
  150 CONTINUE
60105 FORMAT(/'EIGENVECTOR MATRIX')
60151 FORMAT('ROW ',I2/(4D18.10))
      END IF
C.. CHECK EIGENVALUES
      NS=0
      NU=0
      DO 200 I=1,N
      IF (WR(I).LT.-CUT) NS=NS+1
      IF (WR(I).GT.CUT) NU=NU+1
  200 CONTINUE
      IF (KPRINT.GT.0) WRITE(6,60201) NS,NU
60201 FORMAT(/'NUMBER OF STABLE EIGENVALUES ',I2/
     &   'NUMBER OF UNSTABLE EIGENVALUES ',I2)
      IF (NS+NU.LT.N) THEN
      IERR=4
      IF (KPRINT.GT.0) WRITE(6,60999)
60999 FORMAT(/'EXIT DUE TO NONHYPERBOLIC EQUILIBRIUM')
      RETURN
      END IF
C.. ORDER EIGENVALUES
      DO 205 I=N,2,-1
      DO 205 L=1,I-1
      IF (WR(L).GT.WR(L+1)) THEN
      TEMP=WR(L)
      WR(L)=WR(L+1)
      WR(L+1)=TEMP
      TEMP=WI(L)
      WI(L)=WI(L+1)
      WI(L+1)=TEMP
      DO 206 J=1,N
      TEMP=Z(J,L)
      Z(J,L)=Z(J,L+1)
      Z(J,L+1)=TEMP
  206 CONTINUE
      END IF
  205 CONTINUE
      IF (KPRINT.GT.1) THEN
      WRITE(6,60203) 
60203 FORMAT(/'ORDERED EIGENVALUES')
      DO 207 I=1,N
      WRITE(6,60131) I,WR(I),WI(I)
  207 CONTINUE
      WRITE(6,60204) 
60204 FORMAT(/'ORDERED EIGENVECTOR MATRIX')
      DO 208 I=1,N
      WRITE(6,60141) I,(Z(I,J),J=1,N)
  208 CONTINUE
      END IF
C.. INVERT EIGENVECTOR MATRIX
      DO 220 I=1,N
      DO 210 J=1,N
      A(I,J)=Z(I,J)
      ZINV(I,J)=0.D0
  210 CONTINUE
      ZINV(I,I)=1.D0
  220 CONTINUE
      CALL DGEFA(A,NMAX,N,IPVT,IERR)
      IF (IERR.NE.0) THEN
      IERR=5
      IF (KPRINT.GT.0) WRITE(6,60907)
60907 FORMAT('INVERSION OF EIGENVECTOR MATRIX FAILED')
      RETURN
      END IF
      DO 230 J=1,N
      CALL DGESL(A,NMAX,N,IPVT,ZINV(1,J),0)
  230 CONTINUE
C.. COMMON ENTRIES
      DO 300 I=1,N
      XFIN(I)=XEQUI(I)
      X0COM(I)=X0(I)
      I0COM(I)=I0(I)
  300 CONTINUE
      IF (R.GE.0.D0) R=0.2D0*WR(NS)
      RCOM=R
      NCOM=N
C.. SOLVE BOUNDARY VALUE PROBLEM BY CONTINUATION
      NCOMP=N
      DO 301 I=1,NCOMP                      
      M(I)=1                                                          
  301 CONTINUE                                                         
      ALEFT=0.D0                                                      
      ARIGHT=1.D0
      IZETA=0
      DO 302 I=1,NU
      IZETA=IZETA+1
      ZETA(IZETA)=ALEFT                                
  302 CONTINUE
      DO 303 I=1,NS
      IZETA=IZETA+1
      ZETA(IZETA)=ARIGHT                   
  303 CONTINUE                                                         
      INFO(1)=1                                                       
      INFO(2)=3                                                       
      INFO(3)=5                                                       
      INFO(8)=0                                                       
      INFO(5)=LRWORK                                    
      INFO(6)=LIWORK                                         
      IF (KPRINT.EQ.0) INFO(7)=1                                  
      IF (KPRINT.EQ.1) INFO(7)=0                                        
      IF (KPRINT.GE.2) INFO(7)=-1            
      INFO(9)=1                                                       
      INFO(10)=0                                                   
      INFO(11)=0                                                      
      INFO(12)=1                                                     
      INFO(13)=1                                                     
      INFO(14)=1                                                     
      INFO(15)=0                                   
      INFO(4)=NCOMP 
      DO 310 I=1,NCOMP                
      LTOL(I)=I
      ZTOL(I)=1.D-5
  310 CONTINUE 
      WINDOW(1)=0.D0
      WINDOW(2)=0.D0
      WINDOW(3)=1.D0
      CALL COLCON(NCOMP,M,ALEFT,ARIGHT,ZETA,INFO,LTOL,     
     &            ZTOL,FIXPNT,WINDOW,IWORK,RWORK,IFLAG,                
     &            FSUB,DFSUB,GSUB,DGSUB,GUESS) 
      IF (IFLAG.NE.1) THEN
      IERR=6
      IF (KPRINT.GT.0) WRITE(6,60399)
60399 FORMAT(/'NO SOLUTION OF BOUNDARY VALUE PROBLEM FOUND')
      RETURN
      END IF
      RETURN
      END
C
C
      SUBROUTINE FCN(N,X,F,IFAIL)
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION X(N),F(N)
      CALL FEVAL(N,X,F) 
      RETURN
      END
C
C
      SUBROUTINE JAC(N,LDJAC,X,DFDX,IFAIL)
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION X(N),DFDX(LDJAC,N)
      CALL JEVAL(N,LDJAC,X,DFDX)                                      
      RETURN
      END      
C
C                                                              
      SUBROUTINE FSUB(S,X,F)                               
      IMPLICIT REAL*8(A-H,O-Z)                                       
      PARAMETER (NMAX=32,NMAX1=NMAX+1)
      DIMENSION X(NMAX1),F(NMAX)                 
      DIMENSION XEQUI(NMAX),X0(NMAX),I0(NMAX)
      DIMENSION WR(NMAX),WI(NMAX),Z(NMAX,NMAX),ZINV(NMAX,NMAX)
      COMMON /COMPRM/ XEQUI,X0,WR,WI,Z,ZINV,R,I0,NS,NU,N
      CALL FEVAL(N,X,F)
      DO 1 I=1,N
      F(I)=F(I)/R/S
    1 CONTINUE 
      RETURN                                                            
      END                                                               
C
C           
      SUBROUTINE DFSUB(S,X,VEC)                             
      IMPLICIT REAL*8(A-H,O-Z)                          
      PARAMETER (NMAX=32,NMAX1=NMAX+1)
      DIMENSION X(NMAX1),VEC(*)           
      DIMENSION XEQUI(NMAX),X0(NMAX),I0(NMAX)
      DIMENSION WR(NMAX),WI(NMAX),Z(NMAX,NMAX),ZINV(NMAX,NMAX)
      COMMON /COMPRM/ XEQUI,X0,WR,WI,Z,ZINV,R,I0,NS,NU,N
      CALL JEVAL(N,N,X,VEC)
      DO 1 I=1,N**2
      VEC(I)=VEC(I)/R/S
    1 CONTINUE 
      RETURN                                                            
      END                                                               
C
C           
      SUBROUTINE GSUB(I,X,G)                               
      IMPLICIT REAL*8(A-H,O-Z)                                         
      PARAMETER (NMAX=32,NMAX1=NMAX+1)
      DIMENSION X(NMAX1)  
      DIMENSION XEQUI(NMAX),X0(NMAX),I0(NMAX)
      DIMENSION WR(NMAX),WI(NMAX),Z(NMAX,NMAX),ZINV(NMAX,NMAX)
      COMMON /COMPRM/ XEQUI,X0,WR,WI,Z,ZINV,R,I0,NS,NU,N
      TAU=X(N+1)
      IF (I.GT.NU) THEN
      IQ=I-NU
      G=X(I0(IQ))-TAU*X0(I0(IQ))-(1.D0-TAU)*XEQUI(I0(IQ))
      ELSE
      G=0.D0
      DO 1 J=1,N
      IQ=I+NS
      G=G+ZINV(IQ,J)*(X(J)-XEQUI(J))
    1 CONTINUE
      END IF      
      RETURN                                                            
      END                                                            
C
C           
      SUBROUTINE DGSUB(I,X,DG)                        
      IMPLICIT REAL*8(A-H,O-Z)                                         
      PARAMETER (NMAX=32,NMAX1=NMAX+1)
      DIMENSION X(NMAX1),DG(NMAX1)                
      DIMENSION XEQUI(NMAX),X0(NMAX),I0(NMAX)
      DIMENSION WR(NMAX),WI(NMAX),Z(NMAX,NMAX),ZINV(NMAX,NMAX)
      COMMON /COMPRM/ XEQUI,X0,WR,WI,Z,ZINV,R,I0,NS,NU,N
      IF (I.GT.NU) THEN
      IQ=I-NU
      DG(I0(IQ))=1.D0
      DG(N+1)=XEQUI(I0(IQ))-X0(I0(IQ))
      ELSE
      IQ=I+NS
      DO 1 J=1,N
      DG(J)=ZINV(IQ,J)
    1 CONTINUE
      END IF
      RETURN                                                            
      END 
C
C           
      SUBROUTINE GUESS(S,X,DMVAL)                          
      IMPLICIT REAL*8(A-H,O-Z)                                         
      PARAMETER (NMAX=32,NMAX1=NMAX+1)
      DIMENSION X(NMAX1),DMVAL(NMAX) 
      DIMENSION XEQUI(NMAX),X0(NMAX),I0(NMAX)
      DIMENSION WR(NMAX),WI(NMAX),Z(NMAX,NMAX),ZINV(NMAX,NMAX)
      COMMON /COMPRM/ XEQUI,X0,WR,WI,Z,ZINV,R,I0,NS,NU,N
      DO 1 I=1,N
      X(I)=XEQUI(I)
      DMVAL(I)=0.D0
    1 CONTINUE
      RETURN                                                            
      END
      
